typedef struct
{
	cl_float2	mStart;
	cl_float2	mEnd;

} clLine2;
DECLARE_NONCOMPLEX_NO_CONSTRUCT_TYPE( clLine2 );

typedef struct
{
	cl_float4	mStart;
	cl_float4	mEnd;

} clLine3;
DECLARE_NONCOMPLEX_NO_CONSTRUCT_TYPE( clLine3 );

typedef struct
{
	clLine2		mLine;
	cl_float	mRadius;
	cl_float	mDummyAlign;

} clCapsule2;
DECLARE_NONCOMPLEX_NO_CONSTRUCT_TYPE( clCapsule2 );

typedef struct
{
	cl_float2	mPos;
	cl_float2	mSize;

} clRect;
DECLARE_NONCOMPLEX_NO_CONSTRUCT_TYPE( clRect );

#define MTX_F4
typedef struct
{
	//	gr: naming from ofMatrix4x4
#if defined(MTX_F4)
	cl_float4	_mat[4];
#else
	cl_float m0x;
	cl_float m0y;
	cl_float m0z;
	cl_float m0w;
	cl_float m1x;
	cl_float m1y;
	cl_float m1z;
	cl_float m1w;
	cl_float m2x;
	cl_float m2y;
	cl_float m2z;
	cl_float m2w;
	cl_float m3x;
	cl_float m3y;
	cl_float m3z;
	cl_float m3w;
#endif

} clMatrix4x4;
DECLARE_NONCOMPLEX_NO_CONSTRUCT_TYPE( clMatrix4x4 );


typedef struct 
{
	clMatrix4x4	mModelViewProjection;
	clMatrix4x4	mInverseModelViewProjection;	//	gr: too complicated to implement this in opencl, so premult!
	//clRect		mViewport;			//	not needed but coupled with the projection matrix

} clCamera;
DECLARE_NONCOMPLEX_NO_CONSTRUCT_TYPE( clCamera );

typedef struct
{
	cl_float2			mPersonWorldSize;
	cl_float			mDistortEstimatedPersons;
	cl_float			mStraightenEstimatedPersons;

} clPersonEstimateParams;
DECLARE_NONCOMPLEX_NO_CONSTRUCT_TYPE( clPersonEstimateParams );

inline clPersonEstimateParams _clPersonEstimateParams()
{
	clPersonEstimateParams PersonParams;
	PersonParams.mDistortEstimatedPersons = 1;
	PersonParams.mStraightenEstimatedPersons = 1;
	PersonParams.mPersonWorldSize = _float2( 0.2f, 1.7f );
	return PersonParams;
}

typedef struct
{
	cl_float2	mMaxDistanceNormal;
	cl_float	mWideBlobFactor;	//	if blobwidth > this*person width, consider for splitting
	cl_int		mRealignPairCapsule;

} clBlobPairParams;
DECLARE_NONCOMPLEX_NO_CONSTRUCT_TYPE( clBlobPairParams );

inline clBlobPairParams _clBlobPairParams()
{
	clBlobPairParams Params;
	Params.mMaxDistanceNormal = _float2( 00.f, 1.1f );
	Params.mWideBlobFactor = 1.7f;
	Params.mRealignPairCapsule = false;
	return Params;
}


typedef struct
{
	cl_float4			mNormal;
	cl_float			mDistance;

} clPlane;
DECLARE_NONCOMPLEX_NO_CONSTRUCT_TYPE( clPlane );

typedef struct
{
	clCapsule2	mCapsule;		//	output capsule
	cl_int		mBlobA;			//	index of a
	cl_int		mBlobB;			//	index of b if applicable (same as A if is split blob)

} clPairedBlob;
DECLARE_NONCOMPLEX_NO_CONSTRUCT_TYPE( clPairedBlob );

#define MAX_BLOBS_IN_PERSON	10
typedef struct
{
	clCapsule2	mCapsule;
	cl_int		mBlobIndex[MAX_BLOBS_IN_PERSON];
	cl_int		mBlobCount;
	cl_int		mPadding[3];

} clBlobPerson;


typedef struct
{
	cl_int4		mLine;		//	startxy	endxy
	cl_float4	mColour;	//	line colour
	cl_float4	mLeftColour;	
	cl_float4	mRightColour;	

} clScanline;
DECLARE_NONCOMPLEX_NO_CONSTRUCT_TYPE( clScanline );


#define clScanlineGroup_MAXSIZE	31	//	+1 must div/4, first is used for base scanline
typedef struct
{
	cl_int		mCount;
	cl_int		mIndexes[clScanlineGroup_MAXSIZE];	//	first index is the scanline index which built this group

} clScanlineGroup;
DECLARE_NONCOMPLEX_NO_CONSTRUCT_TYPE( clScanlineGroup );


//	worst-case needs 4 vertexes per scanline
#define clScanlineContour_MAXVERTS	(clScanlineGroup_MAXSIZE*4)
typedef struct clScanlineContour
{
	cl_float4	mColour;
	cl_int2		mVertexes[clScanlineContour_MAXVERTS];	//	x,y 
	cl_int		mVertexCount;
	cl_int		mError;
	cl_int		mGroup;
	cl_int		mVertexesSkipped;

} clScanlineContour;
DECLARE_NONCOMPLEX_NO_CONSTRUCT_TYPE( clScanlineContour );
